# /*********************************************************************NVMH3****
# Path:  NVSDK\Common\media\cgfx
# File:  anisocg.fx
#
# Copyright NVIDIA Corporation 2002
# TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
# *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
# OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
# BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
# WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
# BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
# ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
# BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#
# Comments:
#       $Id: //sw/devrel6/SDK/MEDIA/CgFX/AnisoCg.fx#1 $
#
# ******************************************************************************/

#
# Ported to PyFX by Calle Lejdfors <calle.lejdfors@cs.lth.se>
#

from FX import *

LightPos = (100.0, 100.0, 100.0, 0.0)

anisoTexture = Texture('Aniso1.tga')


cg = Cg("""
void main(float3 Position : POSITION,
          float4 Normal   : NORMAL,

      out float4 HPosition : POSITION,
      out float4 Color     : COLOR,
      out float4 TexCoord0 : TEXCOORD0,
           
  uniform float4x4 WorldViewProj,
  uniform float4x4 WorldIT,
  uniform float4x4 World,
  uniform float4x4 ViewIT,
  uniform float3   LightPos)
{
    float3 Nn = mul(WorldIT, Normal).xyz;
    Nn = normalize(Nn);
    float4 Po = float4(Position.xyz, 1.0);
    float3 Pw = mul(World, Po).xyz;
    float3 Vn = normalize(ViewIT[3].xyz - Pw);
    float3 Ln = normalize(LightPos - Pw);
    float3 Hn = normalize(Vn + Ln);
    TexCoord0.x = dot(Ln, Nn);
    TexCoord0.y = dot(Hn, Nn);
    HPosition = mul(WorldViewProj, Po);
}
""")

anisoSampler = Sampler(anisoTexture,
                       MinFilter = GL_LINEAR,
                       MagFilter = GL_LINEAR,
                       WrapS  = GL_CLAMP,
                       WrapT  = GL_CLAMP,
                       TextureOp = GL_REPLACE)


technique = [Render(VertexShader = cg(),
                    ZEnable = True,
                    ZWriteEnable = True,
                    AlphaBlendEnable = False,
                    Lighting = False,
                    Texture0 = anisoSampler)]
