# /*********************************************************************NVMH3****
# Path:  NVSDK\Common\media\cgfx
# File:  $Id: //sw/devrel6/SDK/MEDIA/CgFX/durer.fx#1 $
#
# Copyright NVIDIA Corporation 2002
# TO THE MAXIMUM EXTENT PERMITTED BY APPLICABLE LAW, THIS SOFTWARE IS PROVIDED
# *AS IS* AND NVIDIA AND ITS SUPPLIERS DISCLAIM ALL WARRANTIES, EITHER EXPRESS
# OR IMPLIED, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
# AND FITNESS FOR A PARTICULAR PURPOSE.  IN NO EVENT SHALL NVIDIA OR ITS SUPPLIERS
# BE LIABLE FOR ANY SPECIAL, INCIDENTAL, INDIRECT, OR CONSEQUENTIAL DAMAGES
# WHATSOEVER (INCLUDING, WITHOUT LIMITATION, DAMAGES FOR LOSS OF BUSINESS PROFITS,
# BUSINESS INTERRUPTION, LOSS OF BUSINESS INFORMATION, OR ANY OTHER PECUNIARY LOSS)
# ARISING OUT OF THE USE OF OR INABILITY TO USE THIS SOFTWARE, EVEN IF NVIDIA HAS
# BEEN ADVISED OF THE POSSIBILITY OF SUCH DAMAGES.
#
#
# Comments:
#       * Emulation of Durer-style split-tone technique.
#       * Lines are drawn in black for shadow, then light-tone opaque lines
#           are overlaid in a different direction for the highlights.
#
# ******************************************************************************/
#

# Style-changes and porting to PyFX done by Calle Lejdfors <calle.lejdfors@cs.lth.se>.


from FX import *

LightPos = (100.0, 100.0, 100.0, 0.0)
LiteColor = (1.0, 1.0, 1.0 , 1.0)
AmbiColor = (0.17, 0.17, 0.17, 1.0)
SurfColor = (0.4, 0.5, 1.0, 1.0)
Ks = 0.5
SpecExpon = 25.0
OverSample = 1.0
PeriodD = 0.07
PeriodS = 0.07
TrainFrac = 0.5


cg = Cg("""
void vertex(float3 Position     : POSITION,
            float4 UV           : TEXCOORD0,
            float4 Normal       : NORMAL,
            float4 Tangent      : TEXCOORD1,
            float4 Binormal     : TEXCOORD2,
            
        out float4 HPosition    : POSITION,
        out float4 oUV          : TEXCOORD0,
        out float3 LightVec     : TEXCOORD1,
        out float3 WorldNormal  : TEXCOORD2,
        out float3 WorldPos     : TEXCOORD3,
        out float3 WorldView    : TEXCOORD4,
        out float3 WorldTangent : TEXCOORD5,
        out float3 WorldBinorm  : TEXCOORD6,
        out float4 HPos2        : TEXCOORD7,
        
    uniform float4x4 WorldViewProj,
    uniform float4x4 WorldIT,
    uniform float4x4 World,
    uniform float4x4 ViewIT,
    uniform float3 LightPos)
{
    WorldNormal = mul(WorldIT, Normal).xyz;
    WorldTangent = mul(WorldIT, Tangent).xyz;
    WorldBinorm = mul(WorldIT, Binormal).xyz;
    float4 Po = float4(Position.x, Position.y, Position.z, 1.0);
    float3 Pw = mul(World, Po).xyz;
    WorldPos = Pw;
    LightVec = normalize(LightPos - Pw);
    oUV = UV;
    WorldView = normalize(ViewIT[3].xyz - Pw);
    float4 hpos = mul(WorldViewProj, Po);
    HPos2 = hpos;
    HPosition = hpos;
}

void fragment(float4 HPosition    : POSITION,
              float4 UV          : TEXCOORD0,
              float3 LightVec     : TEXCOORD1,
              float3 WorldNormal  : TEXCOORD2,
              float3 WorldPos     : TEXCOORD3,
              float3 WorldView    : TEXCOORD4,
              float3 WorldTangent : TEXCOORD5,
              float3 WorldBinorm  : TEXCOORD6,
              float4 HPos2        : TEXCOORD7,
             
          out float4 color        : COLOR,

      uniform float PeriodD,
      uniform float PeriodS,
      uniform float TrainFrac,
      uniform float SpecExpon,
      uniform float4 AmbiColor,
      uniform float4 SurfColor,
      uniform float4 LiteColor,
      uniform float Ks,
      uniform float OverSample)
{
    float3 Ln = normalize(LightVec);
    float3 Nn = normalize(WorldNormal);
    float3 Vn = normalize(WorldView);
    float3 Hn = normalize(Vn + Ln);
    float ldn = dot(Ln,Nn);
    float hdn = dot(Hn,Nn);
    float4 litV = lit(ldn,hdn,SpecExpon);

    float edge = PeriodD*litV.y;
    float width = abs(ddx(WorldPos.x)) + abs(ddy(WorldPos.x));
    float w = width*OverSample/PeriodD;
    float x0 = WorldPos.x/PeriodD - (w/2.0);
    float x1 = x0 + w;
    float nedge = edge/PeriodD;
    float i0 = (1.0-nedge)*floor(x0) + max(0.0, frac(x0)-nedge);
    float i1 = (1.0-nedge)*floor(x1) + max(0.0, frac(x1)-nedge);
    float s = (i1 - i0)/w;
    s = min(1.0,max(0.0,s));
    float4 diffContrib = lerp(LiteColor*SurfColor,float4(0.0,0.0,0.0,1.0),s);

    edge = PeriodS * litV.y * litV.z * Ks;
    width = abs(ddx(WorldPos.z)) + abs(ddy(WorldPos.z));
    w = width*OverSample/PeriodS;
    x0 = WorldPos.z/PeriodS - (w/2.0);
    x1 = x0 + w;
    nedge = edge/PeriodS;
    i0 = (1.0-nedge)*floor(x0) + max(0.0, frac(x0)-nedge);
    i1 = (1.0-nedge)*floor(x1) + max(0.0, frac(x1)-nedge);
    s = (i1 - i0)/w;
    s = min(1.0,max(0.0,s));
    float4 result = lerp(LiteColor,diffContrib,s);      // hilight is OPAQUE -- just like Durer drawing

    color = result;
}""")


technique = [Render(VertexShader = cg.vertex(),
                    PixelShader = cg.fragment(),
                    ZEnable = True,
                    ZWriteEnable = True,
                    CullMode = False)]
