from FX import *

backBuffer = Sampler(
   DynamicTexture(TEXTURE_RECT))

##backBuffer = Sampler(DynamicTexture())


blackAndWhiteFS = Cg("""
    float3 main(float2 texcoords : TEXCOORD,
                uniform sampler2D Source) : COLOR
    {
       float3 color = tex2D(Source, texcoords).rgb;
       return 0.33*(color.r+color.g+color.b).rrr;
    }
    """)

blackAndWhite = ImageProcessing(backBuffer,
    FragmentShader = blackAndWhiteFS(Source=backBuffer))

colorFlipFS = Cg("""
    float4 main(float2 texcoords : TEXCOORD,
                uniform sampler2D Source) : COLOR
    {
       float4 color = tex2D(Source, texcoords);
       return color.rbga;
    }
    """)

colorFlip = ImageProcessing(backBuffer,
    FragmentShader = colorFlipFS(Source=backBuffer))

colorMapFS = Cg("""
    float4 main(float2 texcoords : TEXCOORD,
                uniform sampler1D RedRamp,
                //uniform sampler1D GreenRamp,
                uniform sampler1D BlueRamp,
                uniform sampler2D Source) : COLOR
    {
       float4 color = tex2D(Source, texcoords);
       float4 result = tex1D(RedRamp,color.r);
       //result += tex1D(GreenRamp,color.g);
       result += tex1D(BlueRamp,color.b);
       return result;
    }
    """)

def colorMap(redRamp,greenRamp,blueRamp):
    return ImageProcessing(backBuffer,
        FragmentShader = colorMapFS(target=arbfp1,
                                    Source=backBuffer,
                                    RedRamp = redRamp,
                                    GreenRamp = greenRamp,
                                    BlueRamp = blueRamp))

def ramp(filename):
    return Sampler(Texture(filename, TEXTURE_1D),
                   WrapS = CLAMP, WrapT = CLAMP,
                   MaxLevel = 0,
                   MinFilter = NEAREST,
                   MagFilter = NEAREST,
                   MipFilter = None
                   )

#technique = [colorMap(ramp('redRamp.png'),
#                      ramp('greenRamp.png'),
#                      ramp('blueRamp.png'))
#             ]


from Numeric import *
from Convolution import *

def convolve3x3(kernel):
    s = sum(sum(kernel))
    return ImageProcessing(backBuffer,
        FragmentShader = convolve3x3FS(Source=backBuffer,
                                       Kernel=kernel,
                                       Sum=s,
                                       Direction0=(1,0),
                                       Direction1=(0,1)
                                       ))

kernel = array([[0.3, 0.3, 0.3],
                [0.3, 0.3, 0.3],
                [0.3, 0.3, 0.3]], Float)


#technique = [convolve3x3(kernel)]

edgeDetectFS = Cg("""
float4 main(float2 texcoord : TEXCOORD,
            uniform samplerRECT Source,
            uniform float2 Direction0,
            uniform float2 Direction1,
            uniform float SquaredThreshold) : COLOR
{
   float2 tc = texcoord + Direction1;
   float4 c00 = texRECT(Source, tc - Direction0);
   float4 c01 = texRECT(Source, tc             );
   float4 c02 = texRECT(Source, tc + Direction0);

   
   float4 c10 = texRECT(Source, texcoord - Direction0 );
   float4 c11 = texRECT(Source, texcoord              );
   float4 c12 = texRECT(Source, texcoord + Direction0 );

   tc = texcoord - Direction1;
   float4 c20 = texRECT(Source, tc - Direction0);
   float4 c21 = texRECT(Source, tc             );
   float4 c22 = texRECT(Source, tc + Direction0);

   float4 sx = c20 + c22 - c00 - c02 + 2*(c21-c01);
   float4 sy = c22 + c02 - c00 - c20 + 2*(c12-c10);

   float4 dist = sx*sx + sy*sy;

   float result = 1.0;
   if ( (dist.x+dist.y+dist.z)/3.0 < SquaredThreshold ) {
      result = 0;
   }
   return result.xxxx;
}
""")

edgeDetect = ImageProcessing(backBuffer,
    FragmentShader = edgeDetectFS(Source=backBuffer,
                                  Direction0 = (1,0),
                                  Direction1 = (0,1),
                                  SquaredThreshold = 0.1))

technique = [Render(),edgeDetect]



