#!/usr/bin/python2.2

from OpenGL.GL import *
from OpenGL.GLU import *
from OpenGL.GLUT import *
import math
from glut_linear import *

def to_radians(s): return (s * 3.14159265) / 180.0
def to_degrees(s): return (s * 180.0) / 3.14159265
def cross(a, b):
	return [a[1]*b[2]-b[1]*a[2], -(a[0]*b[2]-b[0]*a[2]), a[0]*b[1]-b[0]*a[1]]
def dot(a, b):
	return a[0]*b[0] + a[1]*b[1] + a[2]*b[2]
def length(a):
	return sqrt(dot(a, a))
def normalize(a):
	l = length(a)
	return [a[i]/l for i in range(len(a))]

class glut_interactor:
	def __init__(self):
		self.enabled = 1

	def display(self): pass
	def idle(self): pass
	def keyboard(self, key, x, y): pass
	def menu_status(self, status, x, y): pass
	def motion(self, x, y): pass
	def mouse(self, button, state, x, y): pass
	def passive_motion(self, x, y): pass
	def reshape(self, w, h): pass
	def special(self, key, x, y): pass
	def timer(self, value): pass
	def visibility(self, v): pass

	def enable(self): self.enabled = 1
	def disable(self): self.enabled = 0

interactors = []
propagate = 1

def glut_display_function():
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.display()

def glut_idle_function():
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.idle()

def glut_keyboard_function(k, x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.keyboard(k, x, y)

def glut_menu_status_function(status, x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.menu_status(status, x, y)

def glut_motion_function(x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.motion(x, y)

def glut_mouse_function(button, state, x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.mouse(button, state, x, y)

def glut_passive_motion_function(x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.passive_motion(x, y)

def glut_reshape_function(w, h):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.reshape(w, h)

def glut_special_function(k, x, y):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.special(k, x, y)

def glut_timer_function(v):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.timer(v)

def glut_visibility_function(v):
	global propagate
	propagate = 1
	for i in interactors:
		if not propagate: break
		i.visibility(v)

def glut_event_processed():
	global propagate
	propagate = 0

def glut_helpers_initialize():
	glutDisplayFunc(glut_display_function)
	glutIdleFunc(None)
	glutKeyboardFunc(glut_keyboard_function)
	glutMenuStatusFunc(glut_menu_status_function)
	glutMotionFunc(glut_motion_function)
	glutMouseFunc(glut_mouse_function)
	glutPassiveMotionFunc(glut_passive_motion_function)
	glutReshapeFunc(glut_reshape_function)
	glutSpecialFunc(glut_special_function)
	glutVisibilityFunc(glut_visibility_function)


def glut_remove_interactor(i):
	global interactors

	if i in interactors:
		interactors.remove(i)

def glut_add_interactor(i, append=1):
	global interactors

	glut_remove_interactor(i)
	if append:
		interactors.append(i)
	else:
		interactors.insert(-1, i)

def glut_timer(msec, value):
	glutTimerFunc(msec, glut_timer_function, value)

def glut_idle(do_idle):
	if do_idle:
		glutIdleFunc(glut_idle_function)
	else:
		glutIdleFunc(None)

class glut_callbacks(glut_interactor):
	def __init__(self):
		glut_interactor.__init__(self)

		self.display_function = None
		self.idle_function = None
		self.keyboard_function = None
		self.menu_status_function = None
		self.motion_function = None
		self.mouse_function = None
		self.passive_motion_function = None
		self.reshape_function = None
		self.special_function = None
		self.timer_function = None
		self.visibility_function = None
	def display(self): 
		if self.display_function: self.display_function()
	def idle(self):
		if self.idle_function: self.idle_function()
	def keyboard(self, k, x, y):
		if self.keyboard_function: self.keyboard_function(k, x, y)
	def menu_status(self, status, x, y):
		if self.menu_status_function:
			self.menu_status_function(status, x, y)
	def motion(self, x, y):
		if self.motion_function: self.motion_function(x, y)
	def mouse(self, button, state, x, y):
		if self.mouse_function: 
			self.mouse_function(button, state, x, y)
	def passive_motion(self, x, y):
		if self.passive_motion_function:
			self.passive_motion_function(x, y)
	def reshape(self, w, h):
		if self.reshape_function:
			self.reshape_function(w, h)
	def special(self, key, x, y):
		if self.special_function: self.special_function(key, x, y)
	def timer(self, value):
		if self.timer_function: self.timer_function(value)
	def visibility(self, v):
		if self.visibility_function: self.visibility_function(v)

class glut_perspective_reshaper(glut_interactor):
	def __init__(self, infovy = 60.0, inzNear = 0.1, inzFar = 10.0):
		glut_interactor.__init__(self)

		self.fovy = infovy
		self.zNear = inzNear
		self.zFar = inzFar
		self.aspect_factor = 1

	def reshape(self, w, h):
		self.width = w
		self.height = h
		
		if self.enabled: self.apply()

	def apply(self):
		glViewport(0,0, self.width, self.height)
		glMatrixMode(GL_PROJECTION)
		glLoadIdentity()
		self.apply_perspective()
		glMatrixMode(GL_MODELVIEW)

	def apply_perspective(self):
		self.aspect = self.aspect_factor * self.width/(self.height * 1.0)

		if self.aspect < 1:
			fovx = self.fovy;
			real_fov = to_degrees(2 * math.atan(math.tan(to_radians(fovx/2.0))/self.aspect))
			gluPerspective(real_fov, self.aspect, self.zNear, self.zFar)
		else:
			gluPerspective(self.fovy, self.aspect, self.zNear, self.zFar)


class glut_simple_interactor(glut_interactor):
	def __init__(self):
		glut_interactor.__init__(self)
		self.activate_on = GLUT_LEFT_BUTTON
		self.active = 0
		self.use_modifiers = 1
		self.modifiers = 0
		self.width = 0
		self.height = 0
		self.x0 = self.y0 = self.y = self.x = self.dx = self.dy = 0
		
	def mouse(self, button, state, X, Y):
		if self.enabled and button == self.activate_on \
		   and state == GLUT_DOWN and (not self.use_modifiers \
		   or (self.modifiers == glutGetModifiers())):
			self.active = 1
			self.x = self.x0 = X
			self.y = self.y0 = Y
			self.dx = self.dy = 0
		elif self.enabled and button == self.activate_on and state == GLUT_UP:
			if self.dx == 0 and self.dy == 0:
				self.update()
			self.active = 0
			self.dx = self.dy = 0
	
	def motion(self, X, Y):
		if self.enabled and self.active:
			self.dx = X - self.x
			self.dy = self.y - Y
			self.x = X
			self.y = Y
			self.update()
	
	def reshape(self, w, h):
		self.width = w
		self.height = h
	
	def apply_transform(self): pass
	def apply_inverse_transform(self): pass
	def get_transform(self): pass
	def get_inverse_transform(self): pass
	def update(self): pass
	
			
	
class glut_pan(glut_simple_interactor):
	def __init__(self):
		glut_simple_interactor.__init__(self)
		self.scale = 0.01
		self.invert_increment = 0
		self.parent_rotation = None
		self.pan = [0 for i in range(3)]

	def update(self):
		v = [self.dx, self.dy, 0]	
		
		if self.parent_rotation:
			self.parent_rotation.mult_vec(v)

		if self.invert_increment:
			self.pan = [self.pan[i] - v[i] * self.scale for i in range(3)]
		else:
			self.pan = [self.pan[i] + v[i] * self.scale for i in range(3)]		
		glutPostRedisplay()

	def apply_transform(self):
		glTranslatef(self.pan[0], self.pan[1], self.pan[2])

	def apply_inverse_transform(self):
		glTranslatef(-self.pan[0], -self.pan[1], -self.pan[2])

	def get_transform(self):
		m = matrix4()
		m.set_translate(self.pan)
		return m

	def get_inverse_transform(self):
		m = matrix4()
		m.set_translate([-self.pan[i] for i in range(3)])
		return m

class glut_dolly(glut_simple_interactor):
	def __init__(self):
		glut_simple_interactor.__init__(self)
		self.scale = 0.01
		self.invert_increment = 0
		self.parent_rotation = None
		self.dolly = [0, 0, 0]

	def update(self):
		v = [0, 0, self.dy]
		if self.parent_rotation:
			self.parent_rotation.mult_vec(v)

		if self.invert_increment:
			self.dolly = [self.dolly[i] - v[i] * self.scale for i in range(3)]
		else:
			self.dolly = [self.dolly[i] + v[i] * self.scale for i in range(3)]

		glutPostRedisplay()

	def apply_transform(self):
		glTranslatef(self.dolly[0], self.dolly[1], self.dolly[2])

	def apply_inverse_transform(self):
		glTranslatef(-self.dolly[0], -self.dolly[1], -self.dolly[2])

	def get_transform(self):
		m = matrix4()
                m.set_translate(self.dolly)
                return m

        def get_inverse_transform(self):
                m = matrix4()
                m.set_translate([-self.dolly[i] for i in range(3)])
                return m
	
class glut_trackball(glut_simple_interactor):
	def __init__(self):
		glut_simple_interactor.__init__(self)
		self.r = quaternion([0,1,0], 0)
		self.incr = quaternion()
		self.centroid = [0, 0, 0]
		self.scale = -0.01
		self.invert_increment = 0
		self.parent_rotation = None
		self.legacy_mode = 0
		
	def update(self):
		if self.dx == 0 and self.dy == 0:
			self.incr = quaternion()
			return

		if self.legacy_mode:
			v = [self.dy, -self.dx, 0]
			len = length(v)
			v = normalize(v)
			if self.parent_rotation:
				self.parent_rotation.mult_vec(v)
			if self.invert_increment:
				self.incr = quaternion(v, -len * self.scale)
			else:
				self.incr = quaternion(v, len * self.scale)

		else:
			if self.width < self.height:
				min = self.width / 2.0
			else:
				min = self.height / 2.0

			offset = [self.width / 2.0, self.height / 2.0, 0]
			a = [self.x - self.dx, self.height - (self.y + self.dy), 0]
			b = [self.x, self.height - self.y, 0]
			a = [(a[i] - offset[i])/min for i in range(3)]
			b = [(b[i] - offset[i])/min for i in range(3)]
				
			tmpscale = 1.0

			a[2] = pow(2, -0.5 * length(a))
			a = normalize(a)
			b[2] = pow(2, -0.5 * length(b))
			b = normalize(b)
			
			axis = cross(a, b)	
			axis = normalize(axis)			
			angle = acos(a[0]*b[0] + a[1]*b[1] + a[2]*b[2])
			
			if self.parent_rotation:
				self.parent_rotation.mult_vec(axis)
			if self.invert_increment:
				self.incr = quaternion(axis, -angle * tmpscale)
			else:
				self.incr = quaternion(axis, angle * tmpscale)
		
		if self.incr.get_value_q()[3] != 3:
			# incr.mult_right actually multiplies itself
			# and stores the result (dont want to store it,
			# we end up increasing the incr' every step)
			qr = quaternion()
			qr.set(self.incr)
			self.r.set(qr.mult_right(self.r))
		glutPostRedisplay()

	def increment_rotation(self):
		if self.active: return
		if self.incr.get_value_q()[3] != 0:
			qr = quaternion()
			qr.set(self.incr)
			self.r.set(qr.mult_right(self.r))
		glutPostRedisplay()

	def apply_transform(self):
		glTranslatef(self.centroid[0], self.centroid[1], self.centroid[2])
		rotate(self.r)
		glTranslatef(-self.centroid[0], -self.centroid[1], -self.centroid[2])

	def apply_inverse_transform(self):
		glTranslatef(self.centroid[0], self.centroid[1], self.centroid[2])
		rotate(self.r.inverse())
		glTranslatef(-self.centroid[0], -self.centroid[1], -self.centroid[2])

	def get_transform(self):
		mt = matrix4()
		mr = matrix4()
		minvt = matrix4()

		mt.set_translate(self.centroid)
		self.r.get_value(mr)
		minvt.set_translate([-self.centroid[i] for i in range(3)])
		mt.mult_right(mr)
		mt.mult_right(minvt)
		return mt

	def get_inverse_transform(self):
		mt = matrix4()
		mr = matrix4()
		minvt = matrix4()

		mt.set_translate(self.centroid)
		self.r.inverse().get_value(mr)
		minvt.set_translate([-self.centroid[i] for i in range(3)])
		mt.mult_right(mr)
		mt.mult_right(minvt)
		return mt
	
class glut_simple_mouse_interactor(glut_interactor):
	def __init__(self, num_buttons_to_use=3):
		glut_interactor.__init__(self)
		self.pan = glut_pan()
		self.dolly = glut_dolly()
		self.trackball = glut_trackball()

		self.configure_buttons(num_buttons_to_use)
		self.camera_mode = 0

	def enable(self):
		self.pan.enable()
		self.dolly.enable()
		self.trackball.enable()
	
	def disable(self):
		self.pan.disable()
		self.dolly.disable()
		self.trackball.disable()
	
	def set_camera_mode(self, cam):
		self.camera_mode = cam
		if self.camera_mode:
			self.trackball.invert_increment = 1
			self.pan.invert_increment = 1
			self.dolly.invert_increment = 1
			self.pan.parent_rotation = self.trackball.r
			self.dolly.parent_rotation = self.trackball.r
		else:
			self.trackball.invert_increment = 0
			self.pan.invert_increment = 0
			self.dolly.invert_increment = 0
			if self.pan.parent_rotation == self.trackball.r: self.pan.parent_rotation = None
			if self.dolly.parent_rotation == self.trackball.r: self.dolly.parent_rotation = None
		
	def configure_buttons(self, num_buttons_to_use = 3):
		if num_buttons_to_use == 1:
			self.trackball.activate_on = GLUT_LEFT_BUTTON
			self.trackball.modifiers = 0

			self.pan.activate_on = GLUT_LEFT_BUTTON
			self.pan.modifiers = GLUT_ACTIVE_SHIFT

			self.dolly.activate_on = GLUT_LEFT_BUTTON
			self.dolly.modifiers = GLUT_ACTIVE_CTRL

		elif num_buttons_to_use == 2:
			self.trackball.activate_on = GLUT_LEFT_BUTTON
			self.trackball.modifiers = 0

			self.pan.activate_on = GLUT_MIDDLE_BUTTON
			self.pan.modifiers = 0

			self.dolly.activate_on = GLUT_LEFT_BUTTON
			self.dolly.modifiers = GLUT_ACTIVE_CTRL

		else:
			self.trackball.activate_on = GLUT_LEFT_BUTTON;
			self.trackball.modifiers = 0

			self.pan.activate_on = GLUT_MIDDLE_BUTTON
			self.pan.modifiers = 0

			self.dolly.activate_on = GLUT_RIGHT_BUTTON
			self.dolly.modifiers = 0

	def motion(self, x, y):
		self.trackball.motion(x, y)
		self.pan.motion(x, y)
		self.dolly.motion(x, y)

	def mouse(self, button, state, x, y):
		self.trackball.mouse(button, state, x, y)
		self.pan.mouse(button, state, x, y)
		self.dolly.mouse(button, state, x, y)

	def reshape(self, x, y):
		self.trackball.reshape(x, y)
		self.pan.reshape(x, y)
		self.dolly.reshape(x, y)

	def apply_transform(self):
		self.pan.apply_transform()
		self.dolly.apply_transform()
		self.trackball.apply_transform()

	def apply_inverse_transform(self):
		self.trackball.apply_inverse_transform()
		self.dolly.apply_inverse_transform()
		self.pan.apply_inverse_transform()

	def get_transform(self):
		m1 = self.trackball.get_transform()
		m2 = self.dolly.get_transform()
		m3 = self.pan.get_transform()

		m3.mult_right(m2)
		m1.mult_right(m1)
		return m3

	def get_inverse_transform(self):
		m1 = self.trackball.get_inverse_transform()
		m2 = self.dolly.get_inverse_transform()
		m3 = self.pan.get_transform()

		m1.mult_right(m2)
		m1.mult_right(m3)
		return m1

	def set_parent_rotation(self, rp):
		self.trackball.parent_rotation = rp
		self.dolly.parent_rotation = rp
		self.pan.parent_rotation = rp


def rotate(r):
	axis, angle = r.get_value()
	glRotatef(to_degrees(angle), axis[0], axis[1], axis[2])

